import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="AbstractPropertyLogicalTypeOptions")


@_attrs_define
class AbstractPropertyLogicalTypeOptions:
    """Metadata based on logicalType. This property needs to be set in order to set logicalTypeOptions.

    Attributes:
        minimum (Union[Unset, datetime.datetime, int]): logicalType = integer or logicalType = date. The minimum value.
        maximum (Union[Unset, datetime.datetime, int]): logicalType = integer or logicalType = date. The maximum value.
        pattern (Union[Unset, str]): logicalType = string. Regular expression pattern to define valid value. Follows
            regular expression syntax from ECMA-262 (https://262.ecma-international.org/5.1/#sec-15.10.1).
        exclusive_maximum (Union[Unset, bool]): logicalType = integer or logicalType = date. If set to true, all values
            are strictly less than the maximum value (values < maximum). Otherwise, less than or equal to the maximum value
            (values <= maximum).
        exclusive_minimum (Union[Unset, bool]): logicalType = integer or logicalType = date. If set to true, all values
            are strictly greater than the minimum value (values > minimum). Otherwise, greater than or equal to the minimum
            value (values >= minimum).
        multiple_of (Union[Unset, int]): logicalType = integer. Values must be multiples of this number. For example,
            multiple of 5 has valid values 0, 5, 10, -5.
        max_length (Union[Unset, int]): logicalType = string. Maximum length of the string.
        min_length (Union[Unset, int]): logicalType = string. Minimum length of the string.
        format_ (Union[Unset, str]): logicalType = date. How the date should be formatted using DuckDB notation. By
            default, the dates are formatted according to the ISO 8601 standard.
    """

    minimum: Union[Unset, datetime.datetime, int] = UNSET
    maximum: Union[Unset, datetime.datetime, int] = UNSET
    pattern: Union[Unset, str] = UNSET
    exclusive_maximum: Union[Unset, bool] = UNSET
    exclusive_minimum: Union[Unset, bool] = UNSET
    multiple_of: Union[Unset, int] = UNSET
    max_length: Union[Unset, int] = UNSET
    min_length: Union[Unset, int] = UNSET
    format_: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        minimum: Union[Unset, int, str]
        if isinstance(self.minimum, Unset):
            minimum = UNSET
        elif isinstance(self.minimum, datetime.datetime):
            minimum = self.minimum.isoformat()
        else:
            minimum = self.minimum

        maximum: Union[Unset, int, str]
        if isinstance(self.maximum, Unset):
            maximum = UNSET
        elif isinstance(self.maximum, datetime.datetime):
            maximum = self.maximum.isoformat()
        else:
            maximum = self.maximum

        pattern = self.pattern

        exclusive_maximum = self.exclusive_maximum

        exclusive_minimum = self.exclusive_minimum

        multiple_of = self.multiple_of

        max_length = self.max_length

        min_length = self.min_length

        format_ = self.format_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if minimum is not UNSET:
            field_dict["minimum"] = minimum
        if maximum is not UNSET:
            field_dict["maximum"] = maximum
        if pattern is not UNSET:
            field_dict["pattern"] = pattern
        if exclusive_maximum is not UNSET:
            field_dict["exclusiveMaximum"] = exclusive_maximum
        if exclusive_minimum is not UNSET:
            field_dict["exclusiveMinimum"] = exclusive_minimum
        if multiple_of is not UNSET:
            field_dict["multipleOf"] = multiple_of
        if max_length is not UNSET:
            field_dict["maxLength"] = max_length
        if min_length is not UNSET:
            field_dict["minLength"] = min_length
        if format_ is not UNSET:
            field_dict["format"] = format_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_minimum(data: object) -> Union[Unset, datetime.datetime, int]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                minimum_type_0 = isoparse(data)

                return minimum_type_0
            except:  # noqa: E722
                pass
            return cast(Union[Unset, datetime.datetime, int], data)

        minimum = _parse_minimum(d.pop("minimum", UNSET))

        def _parse_maximum(data: object) -> Union[Unset, datetime.datetime, int]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                maximum_type_0 = isoparse(data)

                return maximum_type_0
            except:  # noqa: E722
                pass
            return cast(Union[Unset, datetime.datetime, int], data)

        maximum = _parse_maximum(d.pop("maximum", UNSET))

        pattern = d.pop("pattern", UNSET)

        exclusive_maximum = d.pop("exclusiveMaximum", UNSET)

        exclusive_minimum = d.pop("exclusiveMinimum", UNSET)

        multiple_of = d.pop("multipleOf", UNSET)

        max_length = d.pop("maxLength", UNSET)

        min_length = d.pop("minLength", UNSET)

        format_ = d.pop("format", UNSET)

        abstract_property_logical_type_options = cls(
            minimum=minimum,
            maximum=maximum,
            pattern=pattern,
            exclusive_maximum=exclusive_maximum,
            exclusive_minimum=exclusive_minimum,
            multiple_of=multiple_of,
            max_length=max_length,
            min_length=min_length,
            format_=format_,
        )

        abstract_property_logical_type_options.additional_properties = d
        return abstract_property_logical_type_options

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Price")


@_attrs_define
class Price:
    """
    Attributes:
        price_amount (Union[Unset, float]): Subscription price per unit of measure in `priceUnit`.
        price_currency (Union[Unset, str]): Currency of the subscription price in `priceAmount`.
        price_unit (Union[Unset, str]): The unit of measure for calculating cost. Examples megabyte, gigabyte.
    """

    price_amount: Union[Unset, float] = UNSET
    price_currency: Union[Unset, str] = UNSET
    price_unit: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        price_amount = self.price_amount

        price_currency = self.price_currency

        price_unit = self.price_unit

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if price_amount is not UNSET:
            field_dict["priceAmount"] = price_amount
        if price_currency is not UNSET:
            field_dict["priceCurrency"] = price_currency
        if price_unit is not UNSET:
            field_dict["priceUnit"] = price_unit

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        price_amount = d.pop("priceAmount", UNSET)

        price_currency = d.pop("priceCurrency", UNSET)

        price_unit = d.pop("priceUnit", UNSET)

        price = cls(
            price_amount=price_amount,
            price_currency=price_currency,
            price_unit=price_unit,
        )

        price.additional_properties = d
        return price

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.any_type_type_5 import AnyTypeType5


T = TypeVar("T", bound="CustomProperty")


@_attrs_define
class CustomProperty:
    """
    Attributes:
        property_ (Union[Unset, str]): The name of the key. Names should be in camel-case the same as if they were
            permanent properties in the contract.
        value (Union['AnyTypeType5', None, Unset, bool, float, int, list[Any], str]):
    """

    property_: Union[Unset, str] = UNSET
    value: Union["AnyTypeType5", None, Unset, bool, float, int, list[Any], str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.any_type_type_5 import AnyTypeType5

        property_ = self.property_

        value: Union[None, Unset, bool, dict[str, Any], float, int, list[Any], str]
        if isinstance(self.value, Unset):
            value = UNSET
        elif isinstance(self.value, list):
            value = self.value

        elif isinstance(self.value, AnyTypeType5):
            value = self.value.to_dict()
        else:
            value = self.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if property_ is not UNSET:
            field_dict["property"] = property_
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.any_type_type_5 import AnyTypeType5

        d = dict(src_dict)
        property_ = d.pop("property", UNSET)

        def _parse_value(data: object) -> Union["AnyTypeType5", None, Unset, bool, float, int, list[Any], str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                componentsschemas_any_type_type_4 = cast(list[Any], data)

                return componentsschemas_any_type_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_any_type_type_5 = AnyTypeType5.from_dict(data)

                return componentsschemas_any_type_type_5
            except:  # noqa: E722
                pass
            return cast(Union["AnyTypeType5", None, Unset, bool, float, int, list[Any], str], data)

        value = _parse_value(d.pop("value", UNSET))

        custom_property = cls(
            property_=property_,
            value=value,
        )

        custom_property.additional_properties = d
        return custom_property

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.abstract_collaborator_configuration import AbstractCollaboratorConfiguration


T = TypeVar("T", bound="Collaborator")


@_attrs_define
class Collaborator:
    """Collaborator

    Attributes:
        id (str):
        client_id (str):
        name (Union[Unset, str]): A name given the collaborator. This value is intended to be read by humans. Example:
            My Collaborator.
        label (Union[Unset, str]): A label given the collaborator. This value is intended to refered to in code.
            Example: my-collaborator.
        role (Union[Unset, str]):
        configuration (Union[Unset, AbstractCollaboratorConfiguration]): A configuration of the collaborator. This is an
            object representing key-value pairs that will be available in the cage as environment variables.
    """

    id: str
    client_id: str
    name: Union[Unset, str] = UNSET
    label: Union[Unset, str] = UNSET
    role: Union[Unset, str] = UNSET
    configuration: Union[Unset, "AbstractCollaboratorConfiguration"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        client_id = self.client_id

        name = self.name

        label = self.label

        role = self.role

        configuration: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.configuration, Unset):
            configuration = self.configuration.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "clientId": client_id,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if label is not UNSET:
            field_dict["label"] = label
        if role is not UNSET:
            field_dict["role"] = role
        if configuration is not UNSET:
            field_dict["configuration"] = configuration

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.abstract_collaborator_configuration import AbstractCollaboratorConfiguration

        d = dict(src_dict)
        id = d.pop("id")

        client_id = d.pop("clientId")

        name = d.pop("name", UNSET)

        label = d.pop("label", UNSET)

        role = d.pop("role", UNSET)

        _configuration = d.pop("configuration", UNSET)
        configuration: Union[Unset, AbstractCollaboratorConfiguration]
        if isinstance(_configuration, Unset):
            configuration = UNSET
        else:
            configuration = AbstractCollaboratorConfiguration.from_dict(_configuration)

        collaborator = cls(
            id=id,
            client_id=client_id,
            name=name,
            label=label,
            role=role,
            configuration=configuration,
        )

        collaborator.additional_properties = d
        return collaborator

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="AlgoSource")


@_attrs_define
class AlgoSource:
    """
    Attributes:
        registry (str):
        image (str):
        tag (str):
        secret (Union[Unset, str]): A value that will be interpreted as an ImagePullSecret. An example would be a github
            access token that has at least the scope `packages:read`, base64 encoded as `username:token`. If no value is
            provided no ImagePullSecret will be used when deploying the cage.
    """

    registry: str
    image: str
    tag: str
    secret: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        registry = self.registry

        image = self.image

        tag = self.tag

        secret = self.secret

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "registry": registry,
                "image": image,
                "tag": tag,
            }
        )
        if secret is not UNSET:
            field_dict["secret"] = secret

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        registry = d.pop("registry")

        image = d.pop("image")

        tag = d.pop("tag")

        secret = d.pop("secret", UNSET)

        algo_source = cls(
            registry=registry,
            image=image,
            tag=tag,
            secret=secret,
        )

        algo_source.additional_properties = d
        return algo_source

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.code_provider_role import CodeProviderRole
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.algo_source import AlgoSource
    from ..models.code_provider_settings import CodeProviderSettings
    from ..models.cron import Cron


T = TypeVar("T", bound="CodeProvider")


@_attrs_define
class CodeProvider:
    """Someone who provides code

    Attributes:
        role (Union[Unset, CodeProviderRole]):  Example: CodeProvider.
        settings (Union[Unset, CodeProviderSettings]):
        cron (Union[Unset, list['Cron']]):
        source (Union[Unset, AlgoSource]):
    """

    role: Union[Unset, CodeProviderRole] = UNSET
    settings: Union[Unset, "CodeProviderSettings"] = UNSET
    cron: Union[Unset, list["Cron"]] = UNSET
    source: Union[Unset, "AlgoSource"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        role: Union[Unset, str] = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.value

        settings: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.settings, Unset):
            settings = self.settings.to_dict()

        cron: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.cron, Unset):
            cron = []
            for cron_item_data in self.cron:
                cron_item = cron_item_data.to_dict()
                cron.append(cron_item)

        source: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.source, Unset):
            source = self.source.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if role is not UNSET:
            field_dict["role"] = role
        if settings is not UNSET:
            field_dict["settings"] = settings
        if cron is not UNSET:
            field_dict["cron"] = cron
        if source is not UNSET:
            field_dict["source"] = source

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.algo_source import AlgoSource
        from ..models.code_provider_settings import CodeProviderSettings
        from ..models.cron import Cron

        d = dict(src_dict)
        _role = d.pop("role", UNSET)
        role: Union[Unset, CodeProviderRole]
        if isinstance(_role, Unset):
            role = UNSET
        else:
            role = CodeProviderRole(_role)

        _settings = d.pop("settings", UNSET)
        settings: Union[Unset, CodeProviderSettings]
        if isinstance(_settings, Unset):
            settings = UNSET
        else:
            settings = CodeProviderSettings.from_dict(_settings)

        cron = []
        _cron = d.pop("cron", UNSET)
        for cron_item_data in _cron or []:
            cron_item = Cron.from_dict(cron_item_data)

            cron.append(cron_item)

        _source = d.pop("source", UNSET)
        source: Union[Unset, AlgoSource]
        if isinstance(_source, Unset):
            source = UNSET
        else:
            source = AlgoSource.from_dict(_source)

        code_provider = cls(
            role=role,
            settings=settings,
            cron=cron,
            source=source,
        )

        code_provider.additional_properties = d
        return code_provider

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.data_quality_sql_type import DataQualitySqlType
from ..types import UNSET, Unset

T = TypeVar("T", bound="DataQualitySql")


@_attrs_define
class DataQualitySql:
    """
    Attributes:
        query (str): Query string that adheres to the dialect of the provided server.
        type_ (Union[Unset, DataQualitySqlType]): The type of quality check. 'text' is human-readable text that
            describes the quality of the data. 'library' is a set of maintained predefined quality attributes such as row
            count or unique. 'sql' is an individual SQL query that returns a value that can be compared. 'custom' is quality
            attributes that are vendor-specific, such as Soda or Great Expectations. Default: DataQualitySqlType.LIBRARY.
    """

    query: str
    type_: Union[Unset, DataQualitySqlType] = DataQualitySqlType.LIBRARY
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        query = self.query

        type_: Union[Unset, str] = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "query": query,
            }
        )
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        query = d.pop("query")

        _type_ = d.pop("type", UNSET)
        type_: Union[Unset, DataQualitySqlType]
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = DataQualitySqlType(_type_)

        data_quality_sql = cls(
            query=query,
            type_=type_,
        )

        data_quality_sql.additional_properties = d
        return data_quality_sql

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CustomServer")


@_attrs_define
class CustomServer:
    """Customise how data is fetched

    Attributes:
        type_ (str): Type of the server. Example: custom.
        location (Union[Unset, str]): A URL to a location.
        path (Union[Unset, str]): Relative or absolute path to the data file(s).
        account (Union[Unset, str]): Account used by the server.
        catalog (Union[Unset, str]): Name of the catalog.
        database (Union[Unset, str]): Name of the database.
        dataset (Union[Unset, str]): Name of the dataset.
        delimiter (Union[Unset, str]): Delimiter.
        endpoint_url (Union[Unset, str]): Server endpoint.
        format_ (Union[Unset, str]): File format.
        host (Union[Unset, str]): Host name or IP address.
        port (Union[Unset, int]): Port to the server. No default value is assumed for custom servers.
        project (Union[Unset, str]): Project name.
        region (Union[Unset, str]): Cloud region.
        region_name (Union[Unset, str]): Region name.
        schema (Union[Unset, str]): Name of the schema.
        service_name (Union[Unset, str]): Name of the service.
        staging_dir (Union[Unset, str]): Staging directory.
        warehouse (Union[Unset, str]): Name of the cluster or warehouse.
    """

    type_: str
    location: Union[Unset, str] = UNSET
    path: Union[Unset, str] = UNSET
    account: Union[Unset, str] = UNSET
    catalog: Union[Unset, str] = UNSET
    database: Union[Unset, str] = UNSET
    dataset: Union[Unset, str] = UNSET
    delimiter: Union[Unset, str] = UNSET
    endpoint_url: Union[Unset, str] = UNSET
    format_: Union[Unset, str] = UNSET
    host: Union[Unset, str] = UNSET
    port: Union[Unset, int] = UNSET
    project: Union[Unset, str] = UNSET
    region: Union[Unset, str] = UNSET
    region_name: Union[Unset, str] = UNSET
    schema: Union[Unset, str] = UNSET
    service_name: Union[Unset, str] = UNSET
    staging_dir: Union[Unset, str] = UNSET
    warehouse: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        location = self.location

        path = self.path

        account = self.account

        catalog = self.catalog

        database = self.database

        dataset = self.dataset

        delimiter = self.delimiter

        endpoint_url = self.endpoint_url

        format_ = self.format_

        host = self.host

        port = self.port

        project = self.project

        region = self.region

        region_name = self.region_name

        schema = self.schema

        service_name = self.service_name

        staging_dir = self.staging_dir

        warehouse = self.warehouse

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
            }
        )
        if location is not UNSET:
            field_dict["location"] = location
        if path is not UNSET:
            field_dict["path"] = path
        if account is not UNSET:
            field_dict["account"] = account
        if catalog is not UNSET:
            field_dict["catalog"] = catalog
        if database is not UNSET:
            field_dict["database"] = database
        if dataset is not UNSET:
            field_dict["dataset"] = dataset
        if delimiter is not UNSET:
            field_dict["delimiter"] = delimiter
        if endpoint_url is not UNSET:
            field_dict["endpointUrl"] = endpoint_url
        if format_ is not UNSET:
            field_dict["format"] = format_
        if host is not UNSET:
            field_dict["host"] = host
        if port is not UNSET:
            field_dict["port"] = port
        if project is not UNSET:
            field_dict["project"] = project
        if region is not UNSET:
            field_dict["region"] = region
        if region_name is not UNSET:
            field_dict["regionName"] = region_name
        if schema is not UNSET:
            field_dict["schema"] = schema
        if service_name is not UNSET:
            field_dict["serviceName"] = service_name
        if staging_dir is not UNSET:
            field_dict["stagingDir"] = staging_dir
        if warehouse is not UNSET:
            field_dict["warehouse"] = warehouse

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = d.pop("type")

        location = d.pop("location", UNSET)

        path = d.pop("path", UNSET)

        account = d.pop("account", UNSET)

        catalog = d.pop("catalog", UNSET)

        database = d.pop("database", UNSET)

        dataset = d.pop("dataset", UNSET)

        delimiter = d.pop("delimiter", UNSET)

        endpoint_url = d.pop("endpointUrl", UNSET)

        format_ = d.pop("format", UNSET)

        host = d.pop("host", UNSET)

        port = d.pop("port", UNSET)

        project = d.pop("project", UNSET)

        region = d.pop("region", UNSET)

        region_name = d.pop("regionName", UNSET)

        schema = d.pop("schema", UNSET)

        service_name = d.pop("serviceName", UNSET)

        staging_dir = d.pop("stagingDir", UNSET)

        warehouse = d.pop("warehouse", UNSET)

        custom_server = cls(
            type_=type_,
            location=location,
            path=path,
            account=account,
            catalog=catalog,
            database=database,
            dataset=dataset,
            delimiter=delimiter,
            endpoint_url=endpoint_url,
            format_=format_,
            host=host,
            port=port,
            project=project,
            region=region,
            region_name=region_name,
            schema=schema,
            service_name=service_name,
            staging_dir=staging_dir,
            warehouse=warehouse,
        )

        custom_server.additional_properties = d
        return custom_server

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .abstract_collaborator import AbstractCollaborator
from .abstract_collaborator_configuration import AbstractCollaboratorConfiguration
from .abstract_data_contract import AbstractDataContract
from .abstract_data_contract_api_version import AbstractDataContractApiVersion
from .abstract_data_contract_kind import AbstractDataContractKind
from .abstract_property import AbstractProperty
from .abstract_property_logical_type import AbstractPropertyLogicalType
from .abstract_property_logical_type_options import AbstractPropertyLogicalTypeOptions
from .abstract_schema import AbstractSchema
from .abstract_schema_logical_type import AbstractSchemaLogicalType
from .algo_source import AlgoSource
from .any_type_type_5 import AnyTypeType5
from .api_server import ApiServer
from .authoritative_definition import AuthoritativeDefinition
from .code_provider import CodeProvider
from .code_provider_role import CodeProviderRole
from .code_provider_settings import CodeProviderSettings
from .collaborator import Collaborator
from .cron import Cron
from .custom_property import CustomProperty
from .custom_server import CustomServer
from .data_consumer import DataConsumer
from .data_consumer_role import DataConsumerRole
from .data_consumer_settings import DataConsumerSettings
from .data_contract import DataContract
from .data_contract_dv import DataContractDV
from .data_contract_odcs import DataContractODCS
from .data_provider import DataProvider
from .data_provider_role import DataProviderRole
from .data_provider_settings import DataProviderSettings
from .data_quality_library import DataQualityLibrary
from .data_quality_library_type import DataQualityLibraryType
from .data_quality_sql import DataQualitySql
from .data_quality_sql_type import DataQualitySqlType
from .description import Description
from .discriminator_collaborator import DiscriminatorCollaborator
from .element import Element
from .get_collaborator_response_404 import GetCollaboratorResponse404
from .get_collaborators_response_401 import GetCollaboratorsResponse401
from .get_collaborators_response_404 import GetCollaboratorsResponse404
from .get_data_contract_response_404 import GetDataContractResponse404
from .price import Price
from .property_ import Property
from .s3_server import S3Server
from .s3_server_dv_properties import S3ServerDvProperties
from .s3_server_format import S3ServerFormat
from .schema import Schema
from .timestamps import Timestamps

__all__ = (
    "AbstractCollaborator",
    "AbstractCollaboratorConfiguration",
    "AbstractDataContract",
    "AbstractDataContractApiVersion",
    "AbstractDataContractKind",
    "AbstractProperty",
    "AbstractPropertyLogicalType",
    "AbstractPropertyLogicalTypeOptions",
    "AbstractSchema",
    "AbstractSchemaLogicalType",
    "AlgoSource",
    "AnyTypeType5",
    "ApiServer",
    "AuthoritativeDefinition",
    "CodeProvider",
    "CodeProviderRole",
    "CodeProviderSettings",
    "Collaborator",
    "Cron",
    "CustomProperty",
    "CustomServer",
    "DataConsumer",
    "DataConsumerRole",
    "DataConsumerSettings",
    "DataContract",
    "DataContractDV",
    "DataContractODCS",
    "DataProvider",
    "DataProviderRole",
    "DataProviderSettings",
    "DataQualityLibrary",
    "DataQualityLibraryType",
    "DataQualitySql",
    "DataQualitySqlType",
    "Description",
    "DiscriminatorCollaborator",
    "Element",
    "GetCollaboratorResponse404",
    "GetCollaboratorsResponse401",
    "GetCollaboratorsResponse404",
    "GetDataContractResponse404",
    "Price",
    "Property",
    "S3Server",
    "S3ServerDvProperties",
    "S3ServerFormat",
    "Schema",
    "Timestamps",
)



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.abstract_property_logical_type import AbstractPropertyLogicalType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.abstract_property_logical_type_options import AbstractPropertyLogicalTypeOptions
    from ..models.any_type_type_5 import AnyTypeType5
    from ..models.data_quality_library import DataQualityLibrary
    from ..models.data_quality_sql import DataQualitySql


T = TypeVar("T", bound="AbstractProperty")


@_attrs_define
class AbstractProperty:
    """
    Attributes:
        physical_type (Union[Unset, str]): The physical element data type in the data source. For example, VARCHAR(2),
            DOUBLE, INT. Example: ['table', 'view', 'topic', 'file'].
        description (Union[Unset, str]): Description of the element.
        business_name (Union[Unset, str]): The business name of the element.
        tags (Union[Unset, list[str]]): A list of tags that may be assigned to the elements (object or property); the
            tags keyword may appear at any level.
        quality (Union[Unset, list[Union['DataQualityLibrary', 'DataQualitySql']]]): Data quality rules with all the
            relevant information for rule setup and execution
        examples (Union[Unset, list[Union['AnyTypeType5', None, bool, float, int, list[Any], str]]]): List of sample
            element values.
        primary_key (Union[Unset, bool]): Boolean value specifying whether the element is primary or not. Default is
            false.
        primary_key_position (Union[Unset, int]): If element is a primary key, the position of the primary key element.
            Starts from 1. Example of `account_id, name` being primary key columns, `account_id` has primaryKeyPosition 1
            and `name` primaryKeyPosition 2. Default to -1.
        logical_type (Union[Unset, AbstractPropertyLogicalType]):
        logical_type_options (Union[Unset, AbstractPropertyLogicalTypeOptions]): Metadata based on logicalType. This
            property needs to be set in order to set logicalTypeOptions.
        required (Union[Unset, bool]): Indicates if the element may contain Null values; possible values are true and
            false. Default is false.
        unique (Union[Unset, bool]): Indicates if the element contains unique values; possible values are true and
            false. Default is false.
        partitioned (Union[Unset, bool]): Indicates if the element is partitioned; possible values are true and false.
        partition_key_position (Union[Unset, int]): If element is used for partitioning, the position of the partition
            element. Starts from 1. Example of `country, year` being partition columns, `country` has partitionKeyPosition 1
            and `year` partitionKeyPosition 2. Default to -1.
        classification (Union[Unset, str]): Can be anything, like confidential, restricted, and public to more advanced
            categorization. Some companies like PayPal, use data classification indicating the class of data in the element;
            expected values are 1, 2, 3, 4, or 5. Example: ['confidential', 'restricted', 'public'].
        encrypted_name (Union[Unset, str]): The element name within the dataset that contains the encrypted element
            value. For example, unencrypted element `email_address` might have an encryptedName of `email_address_encrypt`.
        transform_source_objects (Union[Unset, list[str]]): List of objects in the data source used in the
            transformation.
        transform_logic (Union[Unset, str]): Logic used in the element transformation.
        transform_description (Union[Unset, str]): Describes the transform logic in very simple terms.
        critical_data_element (Union[Unset, bool]): True or false indicator; If element is considered a critical data
            element (CDE) then true else false.
    """

    physical_type: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    business_name: Union[Unset, str] = UNSET
    tags: Union[Unset, list[str]] = UNSET
    quality: Union[Unset, list[Union["DataQualityLibrary", "DataQualitySql"]]] = UNSET
    examples: Union[Unset, list[Union["AnyTypeType5", None, bool, float, int, list[Any], str]]] = UNSET
    primary_key: Union[Unset, bool] = UNSET
    primary_key_position: Union[Unset, int] = UNSET
    logical_type: Union[Unset, AbstractPropertyLogicalType] = UNSET
    logical_type_options: Union[Unset, "AbstractPropertyLogicalTypeOptions"] = UNSET
    required: Union[Unset, bool] = UNSET
    unique: Union[Unset, bool] = UNSET
    partitioned: Union[Unset, bool] = UNSET
    partition_key_position: Union[Unset, int] = UNSET
    classification: Union[Unset, str] = UNSET
    encrypted_name: Union[Unset, str] = UNSET
    transform_source_objects: Union[Unset, list[str]] = UNSET
    transform_logic: Union[Unset, str] = UNSET
    transform_description: Union[Unset, str] = UNSET
    critical_data_element: Union[Unset, bool] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.any_type_type_5 import AnyTypeType5
        from ..models.data_quality_library import DataQualityLibrary

        physical_type = self.physical_type

        description = self.description

        business_name = self.business_name

        tags: Union[Unset, list[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        quality: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.quality, Unset):
            quality = []
            for componentsschemas_data_quality_item_data in self.quality:
                componentsschemas_data_quality_item: dict[str, Any]
                if isinstance(componentsschemas_data_quality_item_data, DataQualityLibrary):
                    componentsschemas_data_quality_item = componentsschemas_data_quality_item_data.to_dict()
                else:
                    componentsschemas_data_quality_item = componentsschemas_data_quality_item_data.to_dict()

                quality.append(componentsschemas_data_quality_item)

        examples: Union[Unset, list[Union[None, bool, dict[str, Any], float, int, list[Any], str]]] = UNSET
        if not isinstance(self.examples, Unset):
            examples = []
            for examples_item_data in self.examples:
                examples_item: Union[None, bool, dict[str, Any], float, int, list[Any], str]
                if isinstance(examples_item_data, list):
                    examples_item = examples_item_data

                elif isinstance(examples_item_data, AnyTypeType5):
                    examples_item = examples_item_data.to_dict()
                else:
                    examples_item = examples_item_data
                examples.append(examples_item)

        primary_key = self.primary_key

        primary_key_position = self.primary_key_position

        logical_type: Union[Unset, str] = UNSET
        if not isinstance(self.logical_type, Unset):
            logical_type = self.logical_type.value

        logical_type_options: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.logical_type_options, Unset):
            logical_type_options = self.logical_type_options.to_dict()

        required = self.required

        unique = self.unique

        partitioned = self.partitioned

        partition_key_position = self.partition_key_position

        classification = self.classification

        encrypted_name = self.encrypted_name

        transform_source_objects: Union[Unset, list[str]] = UNSET
        if not isinstance(self.transform_source_objects, Unset):
            transform_source_objects = self.transform_source_objects

        transform_logic = self.transform_logic

        transform_description = self.transform_description

        critical_data_element = self.critical_data_element

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if physical_type is not UNSET:
            field_dict["physicalType"] = physical_type
        if description is not UNSET:
            field_dict["description"] = description
        if business_name is not UNSET:
            field_dict["businessName"] = business_name
        if tags is not UNSET:
            field_dict["tags"] = tags
        if quality is not UNSET:
            field_dict["quality"] = quality
        if examples is not UNSET:
            field_dict["examples"] = examples
        if primary_key is not UNSET:
            field_dict["primaryKey"] = primary_key
        if primary_key_position is not UNSET:
            field_dict["primaryKeyPosition"] = primary_key_position
        if logical_type is not UNSET:
            field_dict["logicalType"] = logical_type
        if logical_type_options is not UNSET:
            field_dict["logicalTypeOptions"] = logical_type_options
        if required is not UNSET:
            field_dict["required"] = required
        if unique is not UNSET:
            field_dict["unique"] = unique
        if partitioned is not UNSET:
            field_dict["partitioned"] = partitioned
        if partition_key_position is not UNSET:
            field_dict["partitionKeyPosition"] = partition_key_position
        if classification is not UNSET:
            field_dict["classification"] = classification
        if encrypted_name is not UNSET:
            field_dict["encryptedName"] = encrypted_name
        if transform_source_objects is not UNSET:
            field_dict["transformSourceObjects"] = transform_source_objects
        if transform_logic is not UNSET:
            field_dict["transformLogic"] = transform_logic
        if transform_description is not UNSET:
            field_dict["transformDescription"] = transform_description
        if critical_data_element is not UNSET:
            field_dict["criticalDataElement"] = critical_data_element

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.abstract_property_logical_type_options import AbstractPropertyLogicalTypeOptions
        from ..models.any_type_type_5 import AnyTypeType5
        from ..models.data_quality_library import DataQualityLibrary
        from ..models.data_quality_sql import DataQualitySql

        d = dict(src_dict)
        physical_type = d.pop("physicalType", UNSET)

        description = d.pop("description", UNSET)

        business_name = d.pop("businessName", UNSET)

        tags = cast(list[str], d.pop("tags", UNSET))

        quality = []
        _quality = d.pop("quality", UNSET)
        for componentsschemas_data_quality_item_data in _quality or []:

            def _parse_componentsschemas_data_quality_item(
                data: object,
            ) -> Union["DataQualityLibrary", "DataQualitySql"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_data_quality_item_type_0 = DataQualityLibrary.from_dict(data)

                    return componentsschemas_data_quality_item_type_0
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_data_quality_item_type_1 = DataQualitySql.from_dict(data)

                return componentsschemas_data_quality_item_type_1

            componentsschemas_data_quality_item = _parse_componentsschemas_data_quality_item(
                componentsschemas_data_quality_item_data
            )

            quality.append(componentsschemas_data_quality_item)

        examples = []
        _examples = d.pop("examples", UNSET)
        for examples_item_data in _examples or []:

            def _parse_examples_item(data: object) -> Union["AnyTypeType5", None, bool, float, int, list[Any], str]:
                if data is None:
                    return data
                try:
                    if not isinstance(data, list):
                        raise TypeError()
                    componentsschemas_any_type_type_4 = cast(list[Any], data)

                    return componentsschemas_any_type_type_4
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_any_type_type_5 = AnyTypeType5.from_dict(data)

                    return componentsschemas_any_type_type_5
                except:  # noqa: E722
                    pass
                return cast(Union["AnyTypeType5", None, bool, float, int, list[Any], str], data)

            examples_item = _parse_examples_item(examples_item_data)

            examples.append(examples_item)

        primary_key = d.pop("primaryKey", UNSET)

        primary_key_position = d.pop("primaryKeyPosition", UNSET)

        _logical_type = d.pop("logicalType", UNSET)
        logical_type: Union[Unset, AbstractPropertyLogicalType]
        if isinstance(_logical_type, Unset):
            logical_type = UNSET
        else:
            logical_type = AbstractPropertyLogicalType(_logical_type)

        _logical_type_options = d.pop("logicalTypeOptions", UNSET)
        logical_type_options: Union[Unset, AbstractPropertyLogicalTypeOptions]
        if isinstance(_logical_type_options, Unset):
            logical_type_options = UNSET
        else:
            logical_type_options = AbstractPropertyLogicalTypeOptions.from_dict(_logical_type_options)

        required = d.pop("required", UNSET)

        unique = d.pop("unique", UNSET)

        partitioned = d.pop("partitioned", UNSET)

        partition_key_position = d.pop("partitionKeyPosition", UNSET)

        classification = d.pop("classification", UNSET)

        encrypted_name = d.pop("encryptedName", UNSET)

        transform_source_objects = cast(list[str], d.pop("transformSourceObjects", UNSET))

        transform_logic = d.pop("transformLogic", UNSET)

        transform_description = d.pop("transformDescription", UNSET)

        critical_data_element = d.pop("criticalDataElement", UNSET)

        abstract_property = cls(
            physical_type=physical_type,
            description=description,
            business_name=business_name,
            tags=tags,
            quality=quality,
            examples=examples,
            primary_key=primary_key,
            primary_key_position=primary_key_position,
            logical_type=logical_type,
            logical_type_options=logical_type_options,
            required=required,
            unique=unique,
            partitioned=partitioned,
            partition_key_position=partition_key_position,
            classification=classification,
            encrypted_name=encrypted_name,
            transform_source_objects=transform_source_objects,
            transform_logic=transform_logic,
            transform_description=transform_description,
            critical_data_element=critical_data_element,
        )

        abstract_property.additional_properties = d
        return abstract_property

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.abstract_data_contract_api_version import AbstractDataContractApiVersion
from ..models.abstract_data_contract_kind import AbstractDataContractKind
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.api_server import ApiServer
    from ..models.custom_server import CustomServer
    from ..models.description import Description
    from ..models.price import Price
    from ..models.s3_server import S3Server
    from ..models.schema import Schema


T = TypeVar("T", bound="DataContractODCS")


@_attrs_define
class DataContractODCS:
    """
    Attributes:
        id (Union[Unset, str]): A unique identifier used to reduce the risk of dataset name collisions, such as a UUID.
            This is an ODCS property and only has meaning for the data owner and is not used to identify data contracts in
            the Datavillage DCP
        version (Union[Unset, str]): Current version of the data contract.
        name (Union[Unset, str]): Name of the data contract.
        status (Union[Unset, str]): Current status of the dataset. Valid values are `production`, `test`, or
            `development`. Example: ['production', 'test', 'development'].
        api_version (Union[Unset, AbstractDataContractApiVersion]): Version of the standard used to build data contract.
            Default value is v3.0.0. Default: AbstractDataContractApiVersion.V3_0_0.
        kind (Union[Unset, AbstractDataContractKind]):  Default: AbstractDataContractKind.DATACONTRACT.
        tenant (Union[Unset, str]): Indicates the property the data is primarily associated with. Value is case
            insensitive.
        tags (Union[Unset, list[str]]): A list of tags that may be assigned to the elements (object or property); the
            tags keyword may appear at any level.
        data_product (Union[Unset, str]): The name of the data product.
        description (Union[Unset, Description]):
        price (Union[Unset, Price]):
        domain (Union[Unset, str]): Name of the logical data domain Example: ['imdb_ds_aggregate',
            'receiver_profile_out', 'transaction_profile_out'].
        sla_default_element (Union[Unset, str]): Element (using the element path notation) to do the checks on.
        contract_created_ts (Union[Unset, datetime.datetime]): Timestamp in UTC of when the data contract was created.
        servers (Union[Unset, list[Union['ApiServer', 'CustomServer', 'S3Server']]]): List of servers where the datasets
            reside.
        schema (Union[Unset, list['Schema']]):
    """

    id: Union[Unset, str] = UNSET
    version: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    status: Union[Unset, str] = UNSET
    api_version: Union[Unset, AbstractDataContractApiVersion] = AbstractDataContractApiVersion.V3_0_0
    kind: Union[Unset, AbstractDataContractKind] = AbstractDataContractKind.DATACONTRACT
    tenant: Union[Unset, str] = UNSET
    tags: Union[Unset, list[str]] = UNSET
    data_product: Union[Unset, str] = UNSET
    description: Union[Unset, "Description"] = UNSET
    price: Union[Unset, "Price"] = UNSET
    domain: Union[Unset, str] = UNSET
    sla_default_element: Union[Unset, str] = UNSET
    contract_created_ts: Union[Unset, datetime.datetime] = UNSET
    servers: Union[Unset, list[Union["ApiServer", "CustomServer", "S3Server"]]] = UNSET
    schema: Union[Unset, list["Schema"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.api_server import ApiServer
        from ..models.custom_server import CustomServer

        id = self.id

        version = self.version

        name = self.name

        status = self.status

        api_version: Union[Unset, str] = UNSET
        if not isinstance(self.api_version, Unset):
            api_version = self.api_version.value

        kind: Union[Unset, str] = UNSET
        if not isinstance(self.kind, Unset):
            kind = self.kind.value

        tenant = self.tenant

        tags: Union[Unset, list[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        data_product = self.data_product

        description: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.description, Unset):
            description = self.description.to_dict()

        price: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.price, Unset):
            price = self.price.to_dict()

        domain = self.domain

        sla_default_element = self.sla_default_element

        contract_created_ts: Union[Unset, str] = UNSET
        if not isinstance(self.contract_created_ts, Unset):
            contract_created_ts = self.contract_created_ts.isoformat()

        servers: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.servers, Unset):
            servers = []
            for servers_item_data in self.servers:
                servers_item: dict[str, Any]
                if isinstance(servers_item_data, ApiServer):
                    servers_item = servers_item_data.to_dict()
                elif isinstance(servers_item_data, CustomServer):
                    servers_item = servers_item_data.to_dict()
                else:
                    servers_item = servers_item_data.to_dict()

                servers.append(servers_item)

        schema: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.schema, Unset):
            schema = []
            for schema_item_data in self.schema:
                schema_item = schema_item_data.to_dict()
                schema.append(schema_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if version is not UNSET:
            field_dict["version"] = version
        if name is not UNSET:
            field_dict["name"] = name
        if status is not UNSET:
            field_dict["status"] = status
        if api_version is not UNSET:
            field_dict["apiVersion"] = api_version
        if kind is not UNSET:
            field_dict["kind"] = kind
        if tenant is not UNSET:
            field_dict["tenant"] = tenant
        if tags is not UNSET:
            field_dict["tags"] = tags
        if data_product is not UNSET:
            field_dict["dataProduct"] = data_product
        if description is not UNSET:
            field_dict["description"] = description
        if price is not UNSET:
            field_dict["price"] = price
        if domain is not UNSET:
            field_dict["domain"] = domain
        if sla_default_element is not UNSET:
            field_dict["slaDefaultElement"] = sla_default_element
        if contract_created_ts is not UNSET:
            field_dict["contractCreatedTs"] = contract_created_ts
        if servers is not UNSET:
            field_dict["servers"] = servers
        if schema is not UNSET:
            field_dict["schema"] = schema

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.api_server import ApiServer
        from ..models.custom_server import CustomServer
        from ..models.description import Description
        from ..models.price import Price
        from ..models.s3_server import S3Server
        from ..models.schema import Schema

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        version = d.pop("version", UNSET)

        name = d.pop("name", UNSET)

        status = d.pop("status", UNSET)

        _api_version = d.pop("apiVersion", UNSET)
        api_version: Union[Unset, AbstractDataContractApiVersion]
        if isinstance(_api_version, Unset):
            api_version = UNSET
        else:
            api_version = AbstractDataContractApiVersion(_api_version)

        _kind = d.pop("kind", UNSET)
        kind: Union[Unset, AbstractDataContractKind]
        if isinstance(_kind, Unset):
            kind = UNSET
        else:
            kind = AbstractDataContractKind(_kind)

        tenant = d.pop("tenant", UNSET)

        tags = cast(list[str], d.pop("tags", UNSET))

        data_product = d.pop("dataProduct", UNSET)

        _description = d.pop("description", UNSET)
        description: Union[Unset, Description]
        if isinstance(_description, Unset):
            description = UNSET
        else:
            description = Description.from_dict(_description)

        _price = d.pop("price", UNSET)
        price: Union[Unset, Price]
        if isinstance(_price, Unset):
            price = UNSET
        else:
            price = Price.from_dict(_price)

        domain = d.pop("domain", UNSET)

        sla_default_element = d.pop("slaDefaultElement", UNSET)

        _contract_created_ts = d.pop("contractCreatedTs", UNSET)
        contract_created_ts: Union[Unset, datetime.datetime]
        if isinstance(_contract_created_ts, Unset):
            contract_created_ts = UNSET
        else:
            contract_created_ts = isoparse(_contract_created_ts)

        servers = []
        _servers = d.pop("servers", UNSET)
        for servers_item_data in _servers or []:

            def _parse_servers_item(data: object) -> Union["ApiServer", "CustomServer", "S3Server"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_odcs_server_type_0 = ApiServer.from_dict(data)

                    return componentsschemas_odcs_server_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_odcs_server_type_1 = CustomServer.from_dict(data)

                    return componentsschemas_odcs_server_type_1
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_odcs_server_type_2 = S3Server.from_dict(data)

                return componentsschemas_odcs_server_type_2

            servers_item = _parse_servers_item(servers_item_data)

            servers.append(servers_item)

        schema = []
        _schema = d.pop("schema", UNSET)
        for schema_item_data in _schema or []:
            schema_item = Schema.from_dict(schema_item_data)

            schema.append(schema_item)

        data_contract_odcs = cls(
            id=id,
            version=version,
            name=name,
            status=status,
            api_version=api_version,
            kind=kind,
            tenant=tenant,
            tags=tags,
            data_product=data_product,
            description=description,
            price=price,
            domain=domain,
            sla_default_element=sla_default_element,
            contract_created_ts=contract_created_ts,
            servers=servers,
            schema=schema,
        )

        data_contract_odcs.additional_properties = d
        return data_contract_odcs

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.data_contract_odcs import DataContractODCS


T = TypeVar("T", bound="DataContract")


@_attrs_define
class DataContract:
    """ODCS compliant description of a data contract with metadata fields added by Datavillage DCP

    Attributes:
        id (str): A unique identifier used in the Datavillage DCP to identify a data contract
        client_id (str): Id of the client owning the data contract
        created (datetime.datetime): The date and time when the document was created.
        updated (datetime.datetime): The date and time of the last update to the document.
        data_contract (DataContractODCS):
    """

    id: str
    client_id: str
    created: datetime.datetime
    updated: datetime.datetime
    data_contract: "DataContractODCS"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        client_id = self.client_id

        created = self.created.isoformat()

        updated = self.updated.isoformat()

        data_contract = self.data_contract.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "clientId": client_id,
                "created": created,
                "updated": updated,
                "dataContract": data_contract,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.data_contract_odcs import DataContractODCS

        d = dict(src_dict)
        id = d.pop("id")

        client_id = d.pop("clientId")

        created = isoparse(d.pop("created"))

        updated = isoparse(d.pop("updated"))

        data_contract = DataContractODCS.from_dict(d.pop("dataContract"))

        data_contract = cls(
            id=id,
            client_id=client_id,
            created=created,
            updated=updated,
            data_contract=data_contract,
        )

        data_contract.additional_properties = d
        return data_contract

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.data_provider_role import DataProviderRole
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.api_server import ApiServer
    from ..models.custom_server import CustomServer
    from ..models.data_provider_settings import DataProviderSettings
    from ..models.s3_server import S3Server


T = TypeVar("T", bound="DataProvider")


@_attrs_define
class DataProvider:
    """Someone who provides Data

    Attributes:
        role (Union[Unset, DataProviderRole]):  Example: DataProvider.
        server (Union['ApiServer', 'CustomServer', 'S3Server', Unset]): Data source details of where data is physically
            stored.
        settings (Union[Unset, DataProviderSettings]):
    """

    role: Union[Unset, DataProviderRole] = UNSET
    server: Union["ApiServer", "CustomServer", "S3Server", Unset] = UNSET
    settings: Union[Unset, "DataProviderSettings"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.api_server import ApiServer
        from ..models.custom_server import CustomServer

        role: Union[Unset, str] = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.value

        server: Union[Unset, dict[str, Any]]
        if isinstance(self.server, Unset):
            server = UNSET
        elif isinstance(self.server, ApiServer):
            server = self.server.to_dict()
        elif isinstance(self.server, CustomServer):
            server = self.server.to_dict()
        else:
            server = self.server.to_dict()

        settings: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.settings, Unset):
            settings = self.settings.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if role is not UNSET:
            field_dict["role"] = role
        if server is not UNSET:
            field_dict["server"] = server
        if settings is not UNSET:
            field_dict["settings"] = settings

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.api_server import ApiServer
        from ..models.custom_server import CustomServer
        from ..models.data_provider_settings import DataProviderSettings
        from ..models.s3_server import S3Server

        d = dict(src_dict)
        _role = d.pop("role", UNSET)
        role: Union[Unset, DataProviderRole]
        if isinstance(_role, Unset):
            role = UNSET
        else:
            role = DataProviderRole(_role)

        def _parse_server(data: object) -> Union["ApiServer", "CustomServer", "S3Server", Unset]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_odcs_server_type_0 = ApiServer.from_dict(data)

                return componentsschemas_odcs_server_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_odcs_server_type_1 = CustomServer.from_dict(data)

                return componentsschemas_odcs_server_type_1
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_odcs_server_type_2 = S3Server.from_dict(data)

            return componentsschemas_odcs_server_type_2

        server = _parse_server(d.pop("server", UNSET))

        _settings = d.pop("settings", UNSET)
        settings: Union[Unset, DataProviderSettings]
        if isinstance(_settings, Unset):
            settings = UNSET
        else:
            settings = DataProviderSettings.from_dict(_settings)

        data_provider = cls(
            role=role,
            server=server,
            settings=settings,
        )

        data_provider.additional_properties = d
        return data_provider

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.s3_server_format import S3ServerFormat
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.s3_server_dv_properties import S3ServerDvProperties


T = TypeVar("T", bound="S3Server")


@_attrs_define
class S3Server:
    """Data can be fetched via S3 protocol

    Attributes:
        type_ (str): Type of the server. Example: s3.
        location (str): S3 URL, starting with s3://
        endpoint_url (Union[Unset, str]): The server endpoint for S3-compatible servers, such as MioIO or Google Cloud
            Storage
        format_ (Union[Unset, S3ServerFormat]): Format of files
        delimeter (Union[Unset, str]): (Only for format = json), how multiple json documents are delimited within one
            file
        dv_properties (Union[Unset, S3ServerDvProperties]): Properties for Datavillage DCP
    """

    type_: str
    location: str
    endpoint_url: Union[Unset, str] = UNSET
    format_: Union[Unset, S3ServerFormat] = UNSET
    delimeter: Union[Unset, str] = UNSET
    dv_properties: Union[Unset, "S3ServerDvProperties"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        location = self.location

        endpoint_url = self.endpoint_url

        format_: Union[Unset, str] = UNSET
        if not isinstance(self.format_, Unset):
            format_ = self.format_.value

        delimeter = self.delimeter

        dv_properties: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.dv_properties, Unset):
            dv_properties = self.dv_properties.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
                "location": location,
            }
        )
        if endpoint_url is not UNSET:
            field_dict["endpointUrl"] = endpoint_url
        if format_ is not UNSET:
            field_dict["format"] = format_
        if delimeter is not UNSET:
            field_dict["delimeter"] = delimeter
        if dv_properties is not UNSET:
            field_dict["dvProperties"] = dv_properties

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.s3_server_dv_properties import S3ServerDvProperties

        d = dict(src_dict)
        type_ = d.pop("type")

        location = d.pop("location")

        endpoint_url = d.pop("endpointUrl", UNSET)

        _format_ = d.pop("format", UNSET)
        format_: Union[Unset, S3ServerFormat]
        if isinstance(_format_, Unset):
            format_ = UNSET
        else:
            format_ = S3ServerFormat(_format_)

        delimeter = d.pop("delimeter", UNSET)

        _dv_properties = d.pop("dvProperties", UNSET)
        dv_properties: Union[Unset, S3ServerDvProperties]
        if isinstance(_dv_properties, Unset):
            dv_properties = UNSET
        else:
            dv_properties = S3ServerDvProperties.from_dict(_dv_properties)

        s3_server = cls(
            type_=type_,
            location=location,
            endpoint_url=endpoint_url,
            format_=format_,
            delimeter=delimeter,
            dv_properties=dv_properties,
        )

        s3_server.additional_properties = d
        return s3_server

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Description")


@_attrs_define
class Description:
    """
    Attributes:
        usage (Union[Unset, str]): Intended usage of the dataset.
        purpose (Union[Unset, str]): Purpose of the dataset.
        limitations (Union[Unset, str]): Limitations of the dataset.
    """

    usage: Union[Unset, str] = UNSET
    purpose: Union[Unset, str] = UNSET
    limitations: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        usage = self.usage

        purpose = self.purpose

        limitations = self.limitations

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if usage is not UNSET:
            field_dict["usage"] = usage
        if purpose is not UNSET:
            field_dict["purpose"] = purpose
        if limitations is not UNSET:
            field_dict["limitations"] = limitations

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        usage = d.pop("usage", UNSET)

        purpose = d.pop("purpose", UNSET)

        limitations = d.pop("limitations", UNSET)

        description = cls(
            usage=usage,
            purpose=purpose,
            limitations=limitations,
        )

        description.additional_properties = d
        return description

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="DataContractDV")


@_attrs_define
class DataContractDV:
    """Metadata added to data contracts by Datavillage DCP

    Attributes:
        id (str): A unique identifier used in the Datavillage DCP to identify a data contract
        client_id (str): Id of the client owning the data contract
    """

    id: str
    client_id: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        client_id = self.client_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "clientId": client_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        client_id = d.pop("clientId")

        data_contract_dv = cls(
            id=id,
            client_id=client_id,
        )

        data_contract_dv.additional_properties = d
        return data_contract_dv

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.data_quality_library_type import DataQualityLibraryType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.any_type_type_5 import AnyTypeType5


T = TypeVar("T", bound="DataQualityLibrary")


@_attrs_define
class DataQualityLibrary:
    """
    Attributes:
        rule (str): Define a data quality check based on the predefined rules as per ODCS.
        type_ (Union[Unset, DataQualityLibraryType]): The type of quality check. 'text' is human-readable text that
            describes the quality of the data. 'library' is a set of maintained predefined quality attributes such as row
            count or unique. 'sql' is an individual SQL query that returns a value that can be compared. 'custom' is quality
            attributes that are vendor-specific, such as Soda or Great Expectations. Default:
            DataQualityLibraryType.LIBRARY.
        must_be (Union['AnyTypeType5', None, Unset, bool, float, int, list[Any], str]):
        must_not_be (Union['AnyTypeType5', None, Unset, bool, float, int, list[Any], str]):
        must_be_greater_than (Union[Unset, float]): Must be greater than the value to be valid. It is equivalent to '>'.
        must_be_greater_or_equal_to (Union[Unset, float]): Must be greater than or equal to the value to be valid. It is
            equivalent to '>='.
        must_be_less_than (Union[Unset, float]): Must be less than the value to be valid. It is equivalent to '<'.
        must_be_less_or_equal_to (Union[Unset, float]): Must be less than or equal to the value to be valid. It is
            equivalent to '<='.
        must_be_between (Union[None, Unset, list[float]]): Must be between the two numbers to be valid. Smallest number
            first in the array.
        must_not_be_between (Union[None, Unset, list[float]]): Must not be between the two numbers to be valid. Smallest
            number first in the array.
    """

    rule: str
    type_: Union[Unset, DataQualityLibraryType] = DataQualityLibraryType.LIBRARY
    must_be: Union["AnyTypeType5", None, Unset, bool, float, int, list[Any], str] = UNSET
    must_not_be: Union["AnyTypeType5", None, Unset, bool, float, int, list[Any], str] = UNSET
    must_be_greater_than: Union[Unset, float] = UNSET
    must_be_greater_or_equal_to: Union[Unset, float] = UNSET
    must_be_less_than: Union[Unset, float] = UNSET
    must_be_less_or_equal_to: Union[Unset, float] = UNSET
    must_be_between: Union[None, Unset, list[float]] = UNSET
    must_not_be_between: Union[None, Unset, list[float]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.any_type_type_5 import AnyTypeType5

        rule = self.rule

        type_: Union[Unset, str] = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        must_be: Union[None, Unset, bool, dict[str, Any], float, int, list[Any], str]
        if isinstance(self.must_be, Unset):
            must_be = UNSET
        elif isinstance(self.must_be, list):
            must_be = self.must_be

        elif isinstance(self.must_be, AnyTypeType5):
            must_be = self.must_be.to_dict()
        else:
            must_be = self.must_be

        must_not_be: Union[None, Unset, bool, dict[str, Any], float, int, list[Any], str]
        if isinstance(self.must_not_be, Unset):
            must_not_be = UNSET
        elif isinstance(self.must_not_be, list):
            must_not_be = self.must_not_be

        elif isinstance(self.must_not_be, AnyTypeType5):
            must_not_be = self.must_not_be.to_dict()
        else:
            must_not_be = self.must_not_be

        must_be_greater_than = self.must_be_greater_than

        must_be_greater_or_equal_to = self.must_be_greater_or_equal_to

        must_be_less_than = self.must_be_less_than

        must_be_less_or_equal_to = self.must_be_less_or_equal_to

        must_be_between: Union[None, Unset, list[float]]
        if isinstance(self.must_be_between, Unset):
            must_be_between = UNSET
        elif isinstance(self.must_be_between, list):
            must_be_between = self.must_be_between

        else:
            must_be_between = self.must_be_between

        must_not_be_between: Union[None, Unset, list[float]]
        if isinstance(self.must_not_be_between, Unset):
            must_not_be_between = UNSET
        elif isinstance(self.must_not_be_between, list):
            must_not_be_between = self.must_not_be_between

        else:
            must_not_be_between = self.must_not_be_between

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "rule": rule,
            }
        )
        if type_ is not UNSET:
            field_dict["type"] = type_
        if must_be is not UNSET:
            field_dict["mustBe"] = must_be
        if must_not_be is not UNSET:
            field_dict["mustNotBe"] = must_not_be
        if must_be_greater_than is not UNSET:
            field_dict["mustBeGreaterThan"] = must_be_greater_than
        if must_be_greater_or_equal_to is not UNSET:
            field_dict["mustBeGreaterOrEqualTo"] = must_be_greater_or_equal_to
        if must_be_less_than is not UNSET:
            field_dict["mustBeLessThan"] = must_be_less_than
        if must_be_less_or_equal_to is not UNSET:
            field_dict["mustBeLessOrEqualTo"] = must_be_less_or_equal_to
        if must_be_between is not UNSET:
            field_dict["mustBeBetween"] = must_be_between
        if must_not_be_between is not UNSET:
            field_dict["mustNotBeBetween"] = must_not_be_between

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.any_type_type_5 import AnyTypeType5

        d = dict(src_dict)
        rule = d.pop("rule")

        _type_ = d.pop("type", UNSET)
        type_: Union[Unset, DataQualityLibraryType]
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = DataQualityLibraryType(_type_)

        def _parse_must_be(data: object) -> Union["AnyTypeType5", None, Unset, bool, float, int, list[Any], str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                componentsschemas_any_type_type_4 = cast(list[Any], data)

                return componentsschemas_any_type_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_any_type_type_5 = AnyTypeType5.from_dict(data)

                return componentsschemas_any_type_type_5
            except:  # noqa: E722
                pass
            return cast(Union["AnyTypeType5", None, Unset, bool, float, int, list[Any], str], data)

        must_be = _parse_must_be(d.pop("mustBe", UNSET))

        def _parse_must_not_be(data: object) -> Union["AnyTypeType5", None, Unset, bool, float, int, list[Any], str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                componentsschemas_any_type_type_4 = cast(list[Any], data)

                return componentsschemas_any_type_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_any_type_type_5 = AnyTypeType5.from_dict(data)

                return componentsschemas_any_type_type_5
            except:  # noqa: E722
                pass
            return cast(Union["AnyTypeType5", None, Unset, bool, float, int, list[Any], str], data)

        must_not_be = _parse_must_not_be(d.pop("mustNotBe", UNSET))

        must_be_greater_than = d.pop("mustBeGreaterThan", UNSET)

        must_be_greater_or_equal_to = d.pop("mustBeGreaterOrEqualTo", UNSET)

        must_be_less_than = d.pop("mustBeLessThan", UNSET)

        must_be_less_or_equal_to = d.pop("mustBeLessOrEqualTo", UNSET)

        def _parse_must_be_between(data: object) -> Union[None, Unset, list[float]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                must_be_between_type_0 = cast(list[float], data)

                return must_be_between_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[float]], data)

        must_be_between = _parse_must_be_between(d.pop("mustBeBetween", UNSET))

        def _parse_must_not_be_between(data: object) -> Union[None, Unset, list[float]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                must_not_be_between_type_0 = cast(list[float], data)

                return must_not_be_between_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[float]], data)

        must_not_be_between = _parse_must_not_be_between(d.pop("mustNotBeBetween", UNSET))

        data_quality_library = cls(
            rule=rule,
            type_=type_,
            must_be=must_be,
            must_not_be=must_not_be,
            must_be_greater_than=must_be_greater_than,
            must_be_greater_or_equal_to=must_be_greater_or_equal_to,
            must_be_less_than=must_be_less_than,
            must_be_less_or_equal_to=must_be_less_or_equal_to,
            must_be_between=must_be_between,
            must_not_be_between=must_not_be_between,
        )

        data_quality_library.additional_properties = d
        return data_quality_library

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class AbstractDataContractApiVersion(str, Enum):
    V2_2_0 = "v2.2.0"
    V2_2_1 = "v2.2.1"
    V2_2_2 = "v2.2.2"
    V3_0_0 = "v3.0.0"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.abstract_schema_logical_type import AbstractSchemaLogicalType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.data_quality_library import DataQualityLibrary
    from ..models.data_quality_sql import DataQualitySql
    from ..models.property_ import Property


T = TypeVar("T", bound="Schema")


@_attrs_define
class Schema:
    """Identifiable ODCS description of a schema

    Attributes:
        id (str):
        name (str): Name of the element.
        physical_type (Union[Unset, str]): The physical element data type in the data source. Example: ['table', 'view',
            'topic', 'file'].
        description (Union[Unset, str]): Description of the element.
        business_name (Union[Unset, str]): The business name of the element.
        tags (Union[Unset, list[str]]): A list of tags that may be assigned to the elements (object or property); the
            tags keyword may appear at any level.
        quality (Union[Unset, list[Union['DataQualityLibrary', 'DataQualitySql']]]): Data quality rules with all the
            relevant information for rule setup and execution
        logical_type (Union[Unset, AbstractSchemaLogicalType]): The logical element data type.
        physical_name (Union[Unset, str]): Physical name. Example: ['table_1_2_0'].
        data_granularity_description (Union[Unset, str]): Granular level of the data in the object. Example:
            ['Aggregation by country'].
        properties (Union[Unset, list['Property']]):
    """

    id: str
    name: str
    physical_type: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    business_name: Union[Unset, str] = UNSET
    tags: Union[Unset, list[str]] = UNSET
    quality: Union[Unset, list[Union["DataQualityLibrary", "DataQualitySql"]]] = UNSET
    logical_type: Union[Unset, AbstractSchemaLogicalType] = UNSET
    physical_name: Union[Unset, str] = UNSET
    data_granularity_description: Union[Unset, str] = UNSET
    properties: Union[Unset, list["Property"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.data_quality_library import DataQualityLibrary

        id = self.id

        name = self.name

        physical_type = self.physical_type

        description = self.description

        business_name = self.business_name

        tags: Union[Unset, list[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        quality: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.quality, Unset):
            quality = []
            for componentsschemas_data_quality_item_data in self.quality:
                componentsschemas_data_quality_item: dict[str, Any]
                if isinstance(componentsschemas_data_quality_item_data, DataQualityLibrary):
                    componentsschemas_data_quality_item = componentsschemas_data_quality_item_data.to_dict()
                else:
                    componentsschemas_data_quality_item = componentsschemas_data_quality_item_data.to_dict()

                quality.append(componentsschemas_data_quality_item)

        logical_type: Union[Unset, str] = UNSET
        if not isinstance(self.logical_type, Unset):
            logical_type = self.logical_type.value

        physical_name = self.physical_name

        data_granularity_description = self.data_granularity_description

        properties: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.properties, Unset):
            properties = []
            for properties_item_data in self.properties:
                properties_item = properties_item_data.to_dict()
                properties.append(properties_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "name": name,
            }
        )
        if physical_type is not UNSET:
            field_dict["physicalType"] = physical_type
        if description is not UNSET:
            field_dict["description"] = description
        if business_name is not UNSET:
            field_dict["businessName"] = business_name
        if tags is not UNSET:
            field_dict["tags"] = tags
        if quality is not UNSET:
            field_dict["quality"] = quality
        if logical_type is not UNSET:
            field_dict["logicalType"] = logical_type
        if physical_name is not UNSET:
            field_dict["physicalName"] = physical_name
        if data_granularity_description is not UNSET:
            field_dict["dataGranularityDescription"] = data_granularity_description
        if properties is not UNSET:
            field_dict["properties"] = properties

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.data_quality_library import DataQualityLibrary
        from ..models.data_quality_sql import DataQualitySql
        from ..models.property_ import Property

        d = dict(src_dict)
        id = d.pop("id")

        name = d.pop("name")

        physical_type = d.pop("physicalType", UNSET)

        description = d.pop("description", UNSET)

        business_name = d.pop("businessName", UNSET)

        tags = cast(list[str], d.pop("tags", UNSET))

        quality = []
        _quality = d.pop("quality", UNSET)
        for componentsschemas_data_quality_item_data in _quality or []:

            def _parse_componentsschemas_data_quality_item(
                data: object,
            ) -> Union["DataQualityLibrary", "DataQualitySql"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_data_quality_item_type_0 = DataQualityLibrary.from_dict(data)

                    return componentsschemas_data_quality_item_type_0
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_data_quality_item_type_1 = DataQualitySql.from_dict(data)

                return componentsschemas_data_quality_item_type_1

            componentsschemas_data_quality_item = _parse_componentsschemas_data_quality_item(
                componentsschemas_data_quality_item_data
            )

            quality.append(componentsschemas_data_quality_item)

        _logical_type = d.pop("logicalType", UNSET)
        logical_type: Union[Unset, AbstractSchemaLogicalType]
        if isinstance(_logical_type, Unset):
            logical_type = UNSET
        else:
            logical_type = AbstractSchemaLogicalType(_logical_type)

        physical_name = d.pop("physicalName", UNSET)

        data_granularity_description = d.pop("dataGranularityDescription", UNSET)

        properties = []
        _properties = d.pop("properties", UNSET)
        for properties_item_data in _properties or []:
            properties_item = Property.from_dict(properties_item_data)

            properties.append(properties_item)

        schema = cls(
            id=id,
            name=name,
            physical_type=physical_type,
            description=description,
            business_name=business_name,
            tags=tags,
            quality=quality,
            logical_type=logical_type,
            physical_name=physical_name,
            data_granularity_description=data_granularity_description,
            properties=properties,
        )

        schema.additional_properties = d
        return schema

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DataQualitySqlType(str, Enum):
    CUSTOM = "custom"
    LIBRARY = "library"
    SQL = "sql"
    TEXT = "text"

    def __str__(self) -> str:
        return str(self.value)
